@@ -1,5 +1,5 @@
 /*
- * $Id: display.c,v 1.124 2003/08/02 13:01:07 kerofin Exp $
+ * $Id: display.c,v 1.127 2004/04/27 10:34:30 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2003, the ROX-Filer team.
@@ -40,10 +40,10 @@
 #include "global.h"
 
 #include "main.h"
+#include "filer.h"
 #include "display.h"
 #include "support.h"
 #include "gui_support.h"
-#include "filer.h"
 #include "pixmaps.h"
 #include "menu.h"
 #include "dnd.h"
@@ -92,7 +92,7 @@ void display_init()
 	option_add_int(&o_display_size, "display_icon_size", AUTO_SIZE_ICONS);
 	option_add_int(&o_display_details, "display_details", DETAILS_NONE);
 	option_add_int(&o_display_sort_by, "display_sort_by", SORT_NAME);
-	option_add_int(&o_large_width, "display_large_width", 89);
+	option_add_int(&o_large_width, "display_large_width", 155);
 	option_add_int(&o_small_width, "display_small_width", 250);
 	option_add_int(&o_display_show_hidden, "display_show_hidden", FALSE);
 	option_add_int(&o_display_show_thumbs, "display_show_thumbs", FALSE);
@@ -334,20 +334,32 @@ int sort_by_owner(const void *item1, const void *item2)
 {
 	const DirItem *i1 = (DirItem *) item1;
 	const DirItem *i2 = (DirItem *) item2;
+	const gchar *name1;
+	const gchar *name2;
 
-	return i1->uid < i2->uid ? -1 :
-		i1->uid > i2->uid ? 1 :
-		sort_by_name(item1, item2);
+	if(i1->uid==i2->uid)
+		return sort_by_name(item1, item2);
+
+	name1=user_name(i1->uid);
+	name2=user_name(i2->uid);
+
+	return strcmp(name1, name2);
 }
 
 int sort_by_group(const void *item1, const void *item2)
 {
 	const DirItem *i1 = (DirItem *) item1;
 	const DirItem *i2 = (DirItem *) item2;
+	const gchar *name1;
+	const gchar *name2;
 
-	return i1->gid < i2->gid ? -1 :
-		i1->gid > i2->gid ? 1 :
-		sort_by_name(item1, item2);
+	if(i1->gid==i2->gid)
+		return sort_by_name(item1, item2);
+
+	name1=group_name(i1->gid);
+	name2=group_name(i2->gid);
+
+	return strcmp(name1, name2);
 }
 
 int sort_by_date(const void *item1, const void *item2)
@@ -436,19 +448,36 @@ void display_set_thumbs(FilerWindow *filer_window, gboolean thumbs)
 	filer_create_thumbs(filer_window);
 }
 
+void display_update_hidden(FilerWindow *filer_window)
+{
+	filer_detach_rescan(filer_window);	/* (updates titlebar) */
+
+	display_set_actual_size(filer_window, FALSE);
+}
+
 /* Set the 'Show Hidden' flag for this window */
 void display_set_hidden(FilerWindow *filer_window, gboolean hidden)
 {
 	if (filer_window->show_hidden == hidden)
 		return;
 
+	/*
 	filer_window->show_hidden = hidden;
+	*/
+	filer_set_hidden(filer_window, hidden);
 
-	filer_detach_rescan(filer_window);	/* (updates titlebar) */
-
-	display_set_actual_size(filer_window, FALSE);
+	display_update_hidden(filer_window);
 }
 
+void display_set_filter(FilerWindow *filer_window, FilterType type,
+			const gchar *filter_string)
+{
+	filer_set_filter(filer_window, type, filter_string);
+
+	display_update_hidden(filer_window);
+}
+
+
 /* Highlight (wink or cursor) this item in the filer window. If the item
  * isn't already there but we're scanning then highlight it if it
  * appears later.
