@@ -1,5 +1,5 @@
 /*
- * $Id: main.c,v 1.164 2003/10/26 14:52:35 tal197 Exp $
+ * $Id: main.c,v 1.167 2004/04/13 09:04:06 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2003, the ROX-Filer team.
@@ -71,6 +71,7 @@
 #include "panel.h"
 #include "session.h"
 #include "minibuffer.h"
+#include "xtypes.h"
 
 int number_of_windows = 0;	/* Quit when this reaches 0 again... */
 int to_wakeup_pipe = -1;	/* Write here to get noticed */
@@ -177,6 +178,9 @@ static void soap_add(xmlNodePtr body,
 static void child_died(int signum);
 static void child_died_callback(void);
 static void wake_up_cb(gpointer data, gint source, GdkInputCondition condition);
+#if GTK_CHECK_VERSION(2,2,0)
+static void xrandr_size_change(GdkScreen *screen, gpointer user_data);
+#endif
 
 /****************************************************************
  *			EXTERNAL INTERFACE			*
@@ -208,6 +212,7 @@ int main(int argc, char **argv)
 	/* Get internationalisation up and running. This requires the
 	 * choices system, to discover the user's preferred language.
 	 */
+	xtype_init();
 	choices_init();
 	options_init();
 	i18n_init();
@@ -285,6 +290,11 @@ int main(int argc, char **argv)
 		"class \"Collection\" style : gtk "
 		"\"rox-default-collection-style\"\n");
 
+#if GTK_CHECK_VERSION(2,2,0)
+	g_signal_connect(gdk_screen_get_default(), "size-changed",
+			 G_CALLBACK(xrandr_size_change), NULL);
+#endif
+
 	/* Process each option in turn */
 	while (1)
 	{
@@ -681,3 +691,16 @@ static void wake_up_cb(gpointer data, gint source, GdkInputCondition condition)
 		dnotify_wakeup();
 #endif
 }
+
+#if GTK_CHECK_VERSION(2,2,0)
+static void xrandr_size_change(GdkScreen *screen, gpointer user_data)
+{
+	screen_width = gdk_screen_get_width(screen);
+	screen_height = gdk_screen_get_height(screen);
+
+	gui_store_screen_geometry(screen);
+
+	panel_update_size();
+	pinboard_update_size();
+}
+#endif
