@@ -1,5 +1,5 @@
 /*
- * $Id: panel.c,v 1.109 2003/11/29 17:46:43 kerofin Exp $
+ * $Id: panel.c,v 1.113 2004/04/30 16:36:41 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2003, the ROX-Filer team.
@@ -43,11 +43,11 @@
 #include "gui_support.h"
 #include "diritem.h"
 #include "pixmaps.h"
+#include "filer.h"
 #include "display.h"
 #include "bind.h"
 #include "dnd.h"
 #include "support.h"
-#include "filer.h"
 #include "icon.h"
 #include "run.h"
 #include "appinfo.h"
@@ -193,6 +193,10 @@ static gboolean panel_want_show_text(PanelIcon *pi);
 static void panel_show_menu(GdkEventButton *event, PanelIcon *pi, Panel *panel);
 static void panel_style_changed(void);
 static void motion_may_raise(Panel *panel, int x, int y);
+static void panel_update(Panel *panel);
+static gboolean panel_check_xinerama(void);
+static GList *build_monitor_number(Option *option,
+					xmlNode *node, guchar *label);
 
 
 static GtkWidget *dnd_highlight = NULL; /* (stops flickering) */
@@ -202,6 +206,11 @@ static GtkWidget *dnd_highlight = NULL; /* (stops flickering) */
 #define SHOW_ICON 2
 static Option o_panel_style;
 static Option o_panel_width;
+static Option o_panel_xinerama;
+static Option o_panel_monitor;
+
+static gint panel_monitor = -1;
+GdkRectangle panel_geometry;
 
 static int closing_panel = 0;	/* Don't panel_save; destroying! */
 
@@ -214,7 +223,14 @@ void panel_init(void)
 	option_add_int(&o_panel_style, "panel_style", SHOW_APPS_SMALL);
 	option_add_int(&o_panel_width, "panel_width", 52);
 
+	option_add_int(&o_panel_xinerama, "panel_xinerama", 0);
+	option_add_int(&o_panel_monitor, "panel_monitor", 0);
+
 	option_add_notify(panel_style_changed);
+
+	option_register_widget("monitor-number", build_monitor_number);
+
+	panel_check_xinerama();
 }
 
 /* 'name' may be NULL or "" to remove the panel */
@@ -458,6 +474,18 @@ void panel_mark_used(GdkRegion *used)
 	}
 }
 
+/* On xrandr screen size changes, update all panels */
+void panel_update_size(void)
+{
+	int i;
+
+	for (i = 0; i < PANEL_NUMBER_OF_SIDES; i++)
+	{
+		if (current_panel[i])
+			panel_update(current_panel[i]);
+	}
+}
+
 /****************************************************************
  *			INTERNAL FUNCTIONS			*
  ****************************************************************/
@@ -966,13 +994,14 @@ static gint icon_button_press(GtkWidget *widget,
 static void reposition_panel(GtkWidget *window,
 				GtkAllocation *alloc, Panel *panel)
 {
-	int		x = 0, y = 0;
+	int		x = panel_geometry.x;
+	int		y = panel_geometry.y;
 	PanelSide	side = panel->side;
 
 	if (side == PANEL_LEFT || side == PANEL_RIGHT)
 	{
 		if (side == PANEL_RIGHT)
-			x = screen_width - alloc->width;
+			x += panel_geometry.width - alloc->width;
 
 		if (current_panel[PANEL_TOP])
 		{
@@ -982,7 +1011,7 @@ static void reposition_panel(GtkWidget *window,
 	}
 
 	if (side == PANEL_BOTTOM)
-		y = screen_height - alloc->height;
+		y += panel_geometry.height - alloc->height;
 	
 	gtk_window_move(GTK_WINDOW(panel->window), x, y);
 	gdk_window_move(panel->window->window, x, y);
@@ -1636,12 +1665,12 @@ static void panel_post_resize(GtkWidget *win, GtkRequisition *req, Panel *panel)
 {
 	if (panel->side == PANEL_TOP || panel->side == PANEL_BOTTOM)
 	{
-		req->width = screen_width;
+		req->width = panel_geometry.width;
 		req->height += EDGE_WIDTH;
 	}
 	else
 	{
-		int h = screen_height;
+		int h = panel_geometry.height;
 
 		if (current_panel[PANEL_TOP])
 		{
@@ -1759,6 +1788,26 @@ static void panel_style_changed(void)
 				panel_update(current_panel[i]);
 		}
 	}
+
+	if (o_panel_xinerama.has_changed || o_panel_monitor.has_changed)
+	{
+		if (panel_check_xinerama())
+		{
+			for (i = 0; i < PANEL_NUMBER_OF_SIDES; i++)
+			{
+				if (current_panel[i])
+				{
+					reposition_panel(
+						current_panel[i]->window,
+						&current_panel[i]->
+						    window->allocation,
+						current_panel[i]);
+					gtk_widget_queue_resize(
+						current_panel[i]->window);
+				}
+			}
+		}
+	}
 }
 
 static gboolean draw_panel_edge(GtkWidget *widget, GdkEventExpose *event,
@@ -1768,7 +1817,7 @@ static gboolean draw_panel_edge(GtkWidget *widget, GdkEventExpose *event,
 
 	if (panel->side == PANEL_TOP || panel->side == PANEL_BOTTOM)
 	{
-		width = screen_width;
+		width = panel_geometry.width;
 		height = EDGE_WIDTH;
 
 		x = 0;
@@ -1780,7 +1829,7 @@ static gboolean draw_panel_edge(GtkWidget *widget, GdkEventExpose *event,
 	else
 	{
 		width = EDGE_WIDTH;
-		height = screen_height;
+		height = panel_geometry.height;
 
 		y = 0;
 		if (panel->side == PANEL_RIGHT)
@@ -2048,3 +2097,45 @@ static void panel_drag_leave(GtkWidget	*widget,
 		window_put_just_above(panel->window->window, pinboard);
 	}
 }
+
+static gboolean panel_check_xinerama(void)
+{
+	gint old_monitor = panel_monitor;
+
+	panel_monitor = -1;
+
+	if (o_panel_xinerama.int_value)
+	{
+		if (o_panel_monitor.int_value < n_monitors)
+		{
+			panel_monitor = o_panel_monitor.int_value; 
+		}
+		else
+		{
+			g_warning(_("Xinerama monitor %d unavailable"),
+					o_panel_monitor.int_value); 
+		}
+	}
+
+	if (panel_monitor == -1)
+	{
+		panel_geometry.x = panel_geometry.y = 0;
+		panel_geometry.width = screen_width;
+		panel_geometry.height = screen_height;
+	}
+	else
+	{
+		panel_geometry = monitor_geom[panel_monitor];
+	}
+
+	return old_monitor != panel_monitor;
+}
+
+static GList *build_monitor_number(Option *option, xmlNode *node, guchar *label)
+{
+	GtkObject *adj;
+	
+	adj = gtk_adjustment_new(MAX(0, panel_monitor),
+				0, n_monitors - 1, 1, 10, 1);
+	return build_numentry_base(option, node, label, GTK_ADJUSTMENT(adj));
+}
