@@ -1,5 +1,5 @@
 /*
- * $Id: view_collection.c,v 1.41 2003/06/19 13:50:18 tal197 Exp $
+ * $Id: view_collection.c,v 1.43 2004/04/13 09:04:07 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2003, the ROX-Filer team.
@@ -44,8 +44,8 @@
 #include "options.h"
 
 #include "toolbar.h"	/* for resizing */
-#include "display.h"
 #include "filer.h"
+#include "display.h"
 
 #define MIN_ITEM_WIDTH 64
 
@@ -1052,7 +1052,7 @@ static void view_collection_add_items(ViewIface *view, GPtrArray *items)
 		DirItem *item = (DirItem *) items->pdata[i];
 		char	*leafname = item->leafname;
 
-		if (leafname[0] == '.' && !filer_window->show_hidden)
+		if (!filer_match_filter(filer_window, leafname))
 			continue;
 
 		add_item(view_collection, item);
@@ -1083,7 +1083,7 @@ static void view_collection_update_items(ViewIface *view, GPtrArray *items)
 		const gchar *leafname = item->leafname;
 		int j;
 
-		if (leafname[0] == '.' && filer_window->show_hidden == FALSE)
+		if (!filer_match_filter(filer_window, leafname))
 			continue;
 
 		j = collection_find_item(collection, item,
@@ -1468,8 +1468,8 @@ static void view_collection_autosize(ViewIface *view)
 		h = ICON_HEIGHT * 1.5;
 	n = MAX(n, 2);
 
-	max_x = (o_filer_size_limit.int_value * screen_width) / 100;
-	max_rows = (o_filer_size_limit.int_value * screen_height) / (h * 100);
+	max_x = (o_filer_size_limit.int_value * monitor_width) / 100;
+	max_rows = (o_filer_size_limit.int_value * monitor_height) / (h * 100);
 
 	/* Aim for a size where
 	 * 	   x = r(y + t + h),		(1)
